@@ -9,9 +9,8 @@ from collections import OrderedDict
 from typing import List, Optional
 from typing import Dict, Set, Tuple
 
-import pandas.io.sql
-
-import data_algebra.pandas_model
+import data_algebra
+import data_algebra.data_model
 import data_algebra.near_sql
 import data_algebra.expr_rep
 import data_algebra.util
@@ -19,7 +18,6 @@ import data_algebra.data_ops_types
 import data_algebra.data_ops
 from data_algebra.OrderedSet import OrderedSet
 import data_algebra.op_catalog
-
 from data_algebra.sql_format_options import SQLFormatOptions
 
 # The db_model can be a bit tricky as SQL is represented a few ways depending
@@ -138,7 +136,7 @@ def _db_is_nan_expr(dbmodel, expression):
     subexpr = dbmodel.expr_to_sql(expression.args[0], want_inline_parens=True)
     return (
         "(CASE"
-        + f" WHEN {subexpr} IS NULL THEN TRUE"
+        + f" WHEN {subexpr} IS NULL THEN FALSE"
         + f" WHEN ({subexpr} > 0) AND (-{subexpr} > 0) THEN TRUE"
         + f" WHEN ({subexpr} != {subexpr}) THEN TRUE"
         + f" WHEN ({subexpr} != 0) AND ({subexpr} = -{subexpr}) THEN TRUE"
@@ -781,7 +779,6 @@ class DBModel:
         string_quote: str = "'",
         sql_formatters=None,
         op_replacements=None,
-        local_data_model=None,
         on_start: str = "",
         on_end: str = "",
         on_joiner: str = "AND",
@@ -795,9 +792,7 @@ class DBModel:
         union_all_term_start: str = "(",
         union_all_term_end: str = ")",
     ):
-        if local_data_model is None:
-            local_data_model = data_algebra.pandas_model.default_data_model
-        self.local_data_model = local_data_model
+        self.local_data_model = data_algebra.data_model.default_data_model()
         if sql_formatters is None:
             sql_formatters = {}
         assert identifier_quote != string_quote
@@ -862,7 +857,7 @@ class DBModel:
         else:
             q = str(q)
         assert isinstance(q, str)
-        pandas.io.sql.execute(q, conn)
+        data_algebra.data_model.default_data_model().pd.io.sql.execute(q, conn)
 
     def read_query(self, conn, q):
         """
@@ -876,9 +871,7 @@ class DBModel:
         else:
             q = str(q)
         assert isinstance(q, str)
-        r = pandas.io.sql.read_sql(q, conn)
-        r = self.local_data_model.pd.DataFrame(r)
-        r = r.reset_index(drop=True)
+        r = self.local_data_model.pd.io.sql.read_sql(q, conn)
         return r
 
     def table_exists(self, conn, table_name: str) -> bool:
@@ -919,6 +912,8 @@ class DBModel:
 
         if qualifiers is not None:
             raise ValueError("non-empty qualifiers not yet supported on insert")
+        incoming_data_model = data_algebra.data_model.lookup_data_model_for_dataframe(d)
+        d = incoming_data_model.to_pandas(d)
         if self.table_exists(conn, table_name):
             if not allow_overwrite:
                 raise ValueError("table " + table_name + " already exists")
@@ -927,6 +922,12 @@ class DBModel:
         # Note: the Pandas to_sql() method appears to have SQLite hard-wired into it
         # it refers to sqlite_master
         # this behavior seems to change if sqlalchemy is active
+        # n.b. sqlalchemy tries to insert values on %
+        d = data_algebra.data_model.lookup_data_model_for_dataframe(d).to_pandas(d)
+        if d.shape[1] < 1:
+            # deal with no columns case
+            d = d.copy()
+            d["_index"] = range(d.shape[0])
         d.to_sql(name=table_name, con=conn, index=False)
 
     # noinspection PyMethodMayBeStatic,PyUnusedLocal
@@ -1459,8 +1460,8 @@ class DBModel:
         )
         # /limit columns
         subsql.terms = {
-            k: v
-            for k, v in subsql.terms.items()
+            k: subsql.terms[k]
+            for k in using
             if k not in drop_columns_node.column_deletions
         }
         return subsql
@@ -2328,15 +2329,39 @@ class DBHandle:
     def __enter__(self):
         return self
 
-    # noinspection PyShadowingBuiltins
     def __exit__(self, type, value, traceback):
         self.close()
 
     def read_query(self, q):
         """
-        Return results of query as a Pandas table.
+        Return results of query as a Pandas data frame.
         """
         return self.db_model.read_query(conn=self.conn, q=q)
+    
+    def read_table(self, table_name:str):
+        """
+        Return table as a Pandas data frame.
+
+        :param table_name: table to read
+        """
+        tn = self.db_model.quote_table_name(table_name)
+        return self.read_query(f"SELECT * FROM {tn}")
+    
+    def create_table(self, *, table_name:str, q):
+        """
+        Create table from query.
+
+        :param table_name: table to create
+        :param q: query
+        :return: table description
+        """
+        tn = self.db_model.quote_table_name(table_name)
+        if isinstance(q, data_algebra.data_ops.ViewRepresentation):
+            q = q.to_sql(db_model=self.db_model)
+        else:
+            q = str(q)
+        self.execute(f"CREATE TABLE {tn} AS {q}")
+        return self.describe_table(table_name)
 
     def describe_table(
         self, table_name: str, *, qualifiers=None, row_limit: Optional[int] = 7
@@ -2366,15 +2391,18 @@ class DBHandle:
         """
         self.db_model.drop_table(self.conn, table_name)
 
-    def insert_table(self, d, *, table_name: str, allow_overwrite: bool = False):
+    def insert_table(self, d, *, table_name: str, allow_overwrite: bool = False) -> data_algebra.data_ops.TableDescription:
         """
         Insert a table into the database.
         """
+        incoming_data_model = data_algebra.data_model.lookup_data_model_for_dataframe(d)
+        d = incoming_data_model.to_pandas(d)
         self.db_model.insert_table(
             conn=self.conn, d=d, table_name=table_name, allow_overwrite=allow_overwrite
         )
-        return self.describe_table(table_name)
-
+        res = self.describe_table(table_name)
+        return res
+    
     def to_sql(
         self,
         ops: data_algebra.data_ops.ViewRepresentation,
