@@ -2,7 +2,7 @@
 
 from typing import Any, Dict, List, NamedTuple, Optional, Tuple
 import hashlib
-import data_algebra.pandas_model
+import data_algebra.data_model
 import data_algebra.db_model
 
 
@@ -13,9 +13,9 @@ def hash_data_frame(d) -> str:
     :param d: data frame
     :return: hash code as a string
     """
-    data_algebra.pandas_model.default_data_model.is_appropriate_data_instance(d)
+    data_algebra.data_model.default_data_model().is_appropriate_data_instance(d)
     hash_str = hashlib.sha256(
-        data_algebra.pandas_model.default_data_model.pd.util.hash_pandas_object(d).values
+        data_algebra.data_model.default_data_model().pd.util.hash_pandas_object(d).values
     ).hexdigest()
     return f"{d.shape}_{list(d.columns)}_{hash_str}"
 
@@ -44,7 +44,7 @@ def make_cache_key(
     data_map_keys.sort()
     for k in data_map_keys:
         assert isinstance(k, str)
-        assert data_algebra.pandas_model.default_data_model.is_appropriate_data_instance(data_map[k])
+        assert data_algebra.data_model.default_data_model().is_appropriate_data_instance(data_map[k])
     return EvalKey(
         db_model_name=str(db_model),
         sql=sql,
@@ -74,7 +74,7 @@ class ResultCache:
         """get result from cache, raise KeyError if not present"""
         k = make_cache_key(db_model=db_model, sql=sql, data_map=data_map)
         res = self.result_cache[k]
-        assert data_algebra.pandas_model.default_data_model.is_appropriate_data_instance(res)
+        assert data_algebra.data_model.default_data_model().is_appropriate_data_instance(res)
         return res.copy()
 
     def store(
@@ -86,7 +86,7 @@ class ResultCache:
         res,
     ) -> None:
         """Store result to cache, mark dirty if change."""
-        assert data_algebra.pandas_model.default_data_model.is_appropriate_data_instance(res)
+        assert data_algebra.data_model.default_data_model().is_appropriate_data_instance(res)
         op_key = make_cache_key(db_model=db_model, sql=sql, data_map=data_map)
         try:
             previous = self.result_cache[op_key]
